<?php

if (isset($_SERVER["env"]) && $_SERVER["env"] === 'develop')
{
    require_once __DIR__ . '/conf/confdev.php';
}
else
{
    require_once __DIR__ . '/conf/conf.php';
}

require_once __DIR__ . '/conf/autoloader.php';
require_once __DIR__ . '/php/get.php';
require_once __DIR__ . '/php/post.php';
set_error_handler('errorHandler', E_ALL);
register_shutdown_function('fatalErrorShutdownHandler');
loger::debug("DEBUG",__FILE__,__LINE__,"Request: " . $_SERVER['REQUEST_METHOD'] . " Path: " . $_SERVER['PATH_INFO']);
switch ($_SERVER['REQUEST_METHOD'])
{
    case 'GET':
        switch ($_SERVER['PATH_INFO'])
        {
            case '/Text':
                GET_Text();
                break;
            case '/Menu':
                GET_Menu();
                break;
            case (preg_match('/Parameters\/*/', $_SERVER['PATH_INFO']) ? $_SERVER['PATH_INFO'] : !$_SERVER['PATH_INFO']):
                GET_Parameters(urldecode($_SERVER['REQUEST_URI']));
                break;
            case '/SaveAllSettings':
                GET_SaveAllData();
                break;
            case '/SaveLocalPhonebook':
                GET_SaveLocalPhonebook();
                break;
            case '/SysDump':
                GET_SysDump();
                break;
            case '/SavePersonalizedSettings':
                GET_SavePersonalizedData();
                break;
            case '/Ringtones':
                GET_Ringtones();
                break;
            case '/RemoteOffice':
                GET_RemoteOffice();
                break;
            case '/StorageStatus':
                GET_StorageStatus();
                break;
            case '/Status':
                GET_Status();
                break;
            case '/SysLog':
                GET_SysLog($_SERVER['REQUEST_URI']);
                break;
            case (preg_match('/State\/*/', $_SERVER['PATH_INFO']) ? $_SERVER['PATH_INFO'] : !$_SERVER['PATH_INFO']):
                GET_State($_SERVER['REQUEST_URI']);
                break;
            case '/tcpdump':
                GET_Tcpdump();
                break;
            case '/sipdump':
                GET_SIPdump();
                break;
            case '/NTPSync':
                GET_NtpSync();
                break;
            case '/NetStatus':
                GET_NetStatus();
                break;
            case '/test':
                break;
            case '/Certificates':
                GET_Certificates();
                break;
            case '/InvalidCertificates':
                GET_InvalidCertificates();
                break;
            case '/Pictures':
                GET_Images();
                break;
            case '/CoreDumps':
                GET_CoreDumps();
                break;
            case '/WatchdogStats':
                GET_WatchdogStats();
                break;
            case '/Uptime':
                GET_Uptime();
                break;
            case '/CallHistory':
                GET_CallHistory();
                break;
            case '/ExpertStat':
                GET_ExpertStat();
                break;
            case (preg_match('/admin\/spacfg/', $_SERVER['PATH_INFO']) ? $_SERVER['PATH_INFO'] : !$_SERVER['PATH_INFO']):
                GET_Config($_SERVER['PATH_INFO']);
                break;
            case '/base':
                header('Content-Type: text/xsl');
                echo file_get_contents(FRONTENDPATH . "/conf/base.xsl");
                break;
            default:
                http_response_code(404);
                echo json_encode(array('message' => '#notfound#'));
        }
        break;
    case 'POST':
        switch ($_SERVER['PATH_INFO'])
        {
            case '/Language':
                POST_Language();
                break;
            case '/Parameters':
                POST_Parameters();
                break;
            case '/Login':
                POST_Login();
                break;
            case '/Logout':
                POST_Logout();
                break;
            case '/State':
                POST_State();
                break;
            case '/RestoreSettings':
                if (isset($_FILES) && !empty($_FILES))
                {
                    POST_RestoreSettings($_FILES);
                }
                break;
            case '/RemoteOfficeSync':
                POST_RemoteOfficeSync();
                break;
            case '/RemoteOfficeSave':
                POST_RemoteOfficeSave();
                break;
            case '/LocalPhonebook':
                if (isset($_FILES) && !empty($_FILES))
                {
                    POST_LocalPhonebook($_FILES);
                }
                break;
            case '/LocalPhonebook/ToUSB':
                POST_TransferLocalPhonebook(TRANSFER_TO_USB);
                break;
            case '/LocalPhonebook/FromUSB':
                POST_TransferLocalPhonebook(TRANSFER_FROM_USB);
                break;
            case '/SysDump':
                POST_SysDump();
                break;
            case '/CLI':
                POST_CLI();
                break;
            case '/Ringtones':
                if (isset($_FILES) && !empty($_FILES))
                {
                    POST_Ringtones($_FILES);
                }
                else
                {
                    POST_Ringtone();
                }
                break;
            case '/FirmwareUpdate':
                if (isset($_FILES) && !empty($_FILES))
                {
                    POST_FirmwareUpdate($_FILES);
                }
                else
                {
                    POST_FirmwareUpdate();
                }
                break;
            case '/Status':
                POST_Status();
                break;
            case '/PIN':
                POST_Pin($_SERVER['REQUEST_URI']);
                break;
            case '/Certificates':
                POST_Certificates($_FILES);
                break;
            case '/UploadXML':
                if (isset($_FILES) && !empty($_FILES))
                {
                    POST_UploadXML($_FILES);
                }
                else
                {
                    http_response_code(500);
                }
                break;
            case '/Pictures':
                if (isset($_FILES) && !empty($_FILES))
                {
                    POST_UploadImages($_FILES);
                }
                else
                {
                    POST_DeleteImages();
                }
                break;
            case '/UploadSipCert':
                if (isset($_FILES) && !empty($_FILES))
                {
                    POST_UploadSipCert($_FILES);
                }
                break;
            case '/RemoveSipCert':

                POST_RemoveSipCertificate();

                break;
            case '/UploadSipPrivateKey':
                if (isset($_FILES) && !empty($_FILES))
                {
                    POST_UploadSipPrivateKey($_FILES);
                }
                break;
            case '/RemoveSipPrivateKey':

                POST_RemoveSipPrivateKey();

                break;
            case '/CallHistory':
                POST_CallHistory();
                break;
            case '/events/user':
                POST_Broadsoft_Events_User();
                break;
            default:
                http_response_code(404);
                echo "Not found POST: " . $_SERVER['PATH_INFO'];
        }
        break;
    case 'HEAD':

        switch ($_SERVER['REQUEST_URI'])
        {
            case '/tcpdump':
                if (file_exists(PCAPPATH))
                {
                    sendHeader(200, 'File exists', filesize(PCAPPATH));
                }
                else
                {
                    sendHeader(404, 'File not exists', 0);
                }
                break;
        }
        break;
    case 'DELETE':

        switch ($_SERVER['REQUEST_URI'])
        {
//      case '/sipdump':
//        DELTETE_SipDump();
//        break;
            default:
                sendHeader(404, 'File not exists', 0);
        }
        break;
    default:
        http_response_code(404);
        echo "Not found METHOD: " . $_SERVER['PATH_INFO'];
}

function getCodeMess($_resource, $_mess, $_code = null)
{
    loger::debug("DEBUG",__FILE__,__LINE__,"Response. Resource:" . $_resource . " message:" . $_mess . " code:" . $_code);
    $mess = new message();
    $messresp = $mess->getMessage($_resource, $_mess);
    if (is_null($_code))
        http_response_code($messresp['code']);
    else
        http_response_code($_code);

    return json_encode(array('message' => $messresp['message']));
}

function sendHeader($_code, $_description, $_contentlength)
{
    http_response_code($_code);
    header("Content-Description:" . $_description);
    header("Content-Length: " . $_contentlength);
}

function fatalErrorShutdownHandler()
{

    $last_error = error_get_last();
    if ($last_error['type'] === 64) {
        loger::debug("ERROR",__FILE__,__LINE__,$last_error['message'] . ' --- ' . $last_error['file'] . ' --- Line: ' . $last_error['line']);
    }
    else
    {
        loger::debug("ERROR",__FILE__,__LINE__,$last_error['type'] . ' --- ' . $last_error['message'] . ' --- ' . $last_error['file'] . ' --- Line: ' . $last_error['line']);
        //in case mqtt lock left, we need to get rid of it
        $fp = fopen(MQTT_MUTEX, "w+");
        if (!flock($fp, LOCK_EX|LOCK_NB, $wouldblock))
        {
            if ($wouldblock)
            {
                unlink(MQTT_MUTEX);
                loger::debug("ERROR",__FILE__,__LINE__, "file: ". MQTT_MUTEX . " unlocked");
            }
        }
    }
}

function errorHandler($_errno, $_errstr, $_errfile, $_errline)
{
    $errorhandler=new errorexceptionhandler();
    $errorhandler->getError($_errno, $_errstr, $_errfile, $_errline);
}
